#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "pc110pad.h"

const char *mode_name[]={
	"RAW      bytes as they come out of the hardware",
	"RARE     debounced up/down and absolute x,y",
	"DEBUG    up/down, bounc, recent, transitions, synth, button",
	"PS2      ps2 relative (default)",
};

const char *mode_fields[]={
	"",
	"    x     y touch debounce",
	"touch debou just  recnt trans synth mouse",
	"   x    y buttons",
};

void dump_new_data(int mode, unsigned char data[3])
{
	printf("%02x %02x %02x  ", data[0],data[1],data[2]);
	switch(mode)
	{
		case PC110PAD_RAW:
		{
			break;
		}
		case PC110PAD_RARE:
		{
			int touch=data[0] & 0x80;
			int debounced=data[0] & 0x08;
			int x=((data[0]&0x70)<<4) | data[1];
			int y=((data[0]&0x07)<<8) | data[2];

			printf("%5d %5d %-5s %-5s\n",
				x, y,
				touch?"yes":"no",
				debounced?"down":"up"
			);
			break;
		}
		case PC110PAD_DEBUG:
		{
			int touch	= data[0] & 0x80;
			int debounce	= data[0] & 0x40;
			int just	= data[0] & 0x0f;
			int recent	= data[1] & 0x80;
			int transits	= data[1] & 0x0f;
			int synth	= data[2]>>4;
			int mouse	= data[2] & 0x01;
			printf("%-5s %-5s %-5s %-5s %5d %5d %-5s\n",
				touch?"yes":"no",
				debounce?"down":"up",
				(just==0)?"":
					(just==1)?"up":
					(just==2)?"down":
					"?",
				recent?"yes":"no",
				transits,
				synth,
				mouse?"down":"up"
			);
			break;
		}
		case PC110PAD_PS2:
		{
			int b,x,y;
			b= !!(data[0]&1) * 4 +
			   !!(data[0]&2) * 1 +
			   !!(data[0]&4) * 2;

			if((data[0]&0x0f)==0) b=8; /* glidepoint tap */

			x=   (data[0] & 0x10) ? data[1]-256 : data[1];
			y= -((data[0] & 0x20) ? data[2]-256 : data[2]);
			
			printf("%4d %4d %c%c%c%c\n",
				x,y,
				(b&1)?'1':' ',
				(b&2)?'2':' ',
				(b&4)?'3':' ',
				(b&8)?'T':' '
			);
		}
	}
}


int main(int argc, char *argv[])
{
	int mouse_fd;
	unsigned int odata[]={-1,-1,-1};
	struct pc110pad_params params;
	char *dev;
	int mode;

	if(argc<2 || argc>3)
	{
		fprintf(stderr,"Usage: %s paddev [mode]\n",argv[0]);
		fprintf(stderr,"where mode is one of:\n");
		fprintf(stderr, "	%d  %s\n", PC110PAD_RAW, mode_name[PC110PAD_RAW]);
		fprintf(stderr, "	%d  %s\n", PC110PAD_RARE, mode_name[PC110PAD_RARE]);
		fprintf(stderr, "	%d  %s\n", PC110PAD_DEBUG, mode_name[PC110PAD_DEBUG]);
		fprintf(stderr, "	%d  %s\n", PC110PAD_PS2, mode_name[PC110PAD_PS2]);
		exit(1);
	}
	dev=argv[1];
	mode=(argc==3)?atoi(argv[2]):-1;

	printf("opening port %s\n",dev);
	mouse_fd=open(dev,O_RDONLY | O_NDELAY);
	if(mouse_fd<0)
	{
		perror("open");
		exit(1);
	}

	if(mode<0)
	{
		mode=PC110PAD_PS2;	/* assume it's a real PS/2 mouse */
	}
	else
	{
		printf("setting mode %d = %s\n",mode,mode_name[mode]);
		if(ioctl(mouse_fd, PC110PADIOCGETP, &params))
		{
			perror("GETP");
			exit(1);
		}
		params.mode=mode;
		if(ioctl(mouse_fd, PC110PADIOCSETP, &params))
		{
			perror("SETP");
			exit(1);
		}
	}
	printf("raw data  %s\n",mode_fields[mode]);
		
	for(;;)
	{
		unsigned char data[3];
		int r;

		for(r=0; r<3; )
		{
			int n=read(mouse_fd,data+r,3-r);
			if(n<0)
			{
continue;
				perror("read");
				exit(1);
			}
			r+=n;
		}
		if(data[0]!=odata[0] || data[1]!=odata[1] || data[2]!=odata[2])
		{
			dump_new_data(mode, data);
			odata[0]=data[0]; odata[1]=data[1]; odata[2]=data[2];
		}
	}
	return 0;
}
